
import json, time
import http.client
from heat import ApiTemperature

class HeatController(object):

    def __init__(self, apiTemperature, mqttClient, deviceId):
        self.apiTemperature = apiTemperature
        self.setPoint = None #float temperature
        self.maxWaterTemperature = None
        self.insideTemperature = None
        self.insideTemperatureTs = 0
        self.mqttClient = mqttClient
        self.deviceId = deviceId

    def setWaterTemp(self, temperature):
        temperature = round(temperature)

        if self.maxWaterTemperature != None:
            temperature = min(self.maxWaterTemperature, temperature)

        conn = http.client.HTTPConnection("192.168.10.40", 80, timeout=3) #raspberry pi3
        conn.request("POST", "/heat/setWaterTemperature.php?temperature={}".format(temperature), '')
        response = conn.getresponse()
        if response.status == 200:
            print("Successfully set water temperature to {}".format(temperature))
            jsonObj = {'ts': int(time.time() * 1000), 'tmp': temperature}
            jsonStr = json.dumps(jsonObj)
            publishTopic = "heating/{}/water_temperature".format(self.deviceId)
            self.mqttClient.publish(publishTopic, jsonStr, qos=1, retain=True, content_type='cnq/0.1+json')
        elif response.status != 202: # 202 status code = not necessary (too early) to send same temperature again
            print("Error while setting water temperature to {}".format(temperature))


    def isSetPointSet(self):
        return self.setPoint != None

    def setMaxWaterTemperature(self, waterTemperature):
        self.maxWaterTemperature = waterTemperature

    def setSetPoint(self, setPointTemperature, publish=True):
        if setPointTemperature:
            setPointTemperature = round(setPointTemperature, 2)

        if setPointTemperature and self.setPoint != setPointTemperature:
            print("Setting new setPoint {}".format(setPointTemperature))
            if publish:
                jsonObj = {'ts': int(time.time() * 1000), 'tmp': setPointTemperature}
                jsonStr = json.dumps(jsonObj)
                publishTopic = "heating/{}/program/setpoint".format(self.deviceId)
                self.mqttClient.publish(publishTopic, jsonStr, qos=1, retain=True, content_type='cnq/0.1+json')
        elif not setPointTemperature:
            print("Clearing setPoint")

        self.setPoint = setPointTemperature

    #milliseconds
    def updateInsideTemperature(self, timestamp, temperature):
        if timestamp > self.insideTemperatureTs:
            self.insideTemperatureTs = timestamp
            self.insideTemperature = temperature

    def updateWaterTemperature(self):
        nowMs = int(time.time() * 1000)
        temperatureAgeMs = nowMs - self.insideTemperatureTs

        if temperatureAgeMs < 360000:
            print("Room temp {} measured {} ms ago.".format(self.insideTemperature, temperatureAgeMs))

            outsideTemperature = self.apiTemperature.getTemperature()
            self.adjustWaterTemp(self.insideTemperature, outsideTemperature)
        else:
            print("Room temp is older than 6 minutes. Turning off heating system.")
            self.setWaterTemp(0)


    def adjustWaterTemp(self, roomTemperature, outsideTemperature):
        if self.setPoint == None:
            print("Setpoint is not set yet. Cannot adjust water temperature.")
            return

        tempDiff = self.setPoint - roomTemperature;

        print("Target temp {}, Outside temp {}".format(self.setPoint, outsideTemperature))
        print("Room-Target Difference: {:.2f} °C".format(tempDiff))

        '''
        use variables: current indoor temp, current outdoor temp, current target temp, indoor temp change rate last 3 minutes, indoor temp change rate last 15 minutes
        adjust values depending on feedback (indoor temp change rate) and upon prediction where temp will be in 10/30/x minutes...
        '''

        if tempDiff > 0: #we need to heat
            offset = max(0, abs(self.setPoint-outsideTemperature) - 3)
            print("offset {:.2f}".format(offset))
            waterTemperature = 40 + offset + tempDiff
            if tempDiff < 0.2:
                waterTemperature -= 5

            self.setWaterTemp(waterTemperature)
        elif tempDiff <= 0: #we're too hot
            if abs(tempDiff) < 0.3: #only a little too hot
                waterTemperature = 45
            elif abs(tempDiff) < 0.5:
                waterTemperature = 43
            else:
                waterTemperature = 0

            self.setWaterTemp(waterTemperature)
