
import json, time
from datetime import datetime, timezone, timedelta
from tzlocal import get_localzone

'''
Helps determining the setpoint for a given point in (local) time

Edge case: Sunday 22:00 -> 18°
on Monday 01:00 no entry will be found

algorithm: first search entry for current day.
if nothing found search entry for preceeding day
'''
class Timer(object):

    def __init__(self, timerConfig):
        self.dateTriggers = []

        for timer in timerConfig['timers']:
            spConfigs = []
            days = timer['days']

            for sp in timer['setpoints']:
                spConfigs.append(SetPointConfig(sp['time'], sp['setpoint']))

            spConfigs.sort(reverse=True)
            self.dateTriggers.append(DayConfig(days, spConfigs))


    def getCurrentSetPoint(self, dt):
        if not self.dateTriggers:
            return None

        for dateTrigger in self.dateTriggers:
            setPoint = dateTrigger.getSetPoint(dt)
            if setPoint:
                return setPoint

        """handle edge case: no entry for current day - last entry from last day is valid"""
        naive = dt.replace(tzinfo=None) - timedelta(days=1) # same time

        # this line does not work with winter time (!)
        # error: 'backports.zoneinfo.ZoneInfo' object has no attribute 'localize'
        yesterday = get_localzone().localize(naive, is_dst=None) # but elapsed hours may differ
        yesterday = yesterday.replace(hour=23, minute=59)

        for dateTrigger in self.dateTriggers:
            setPoint = dateTrigger.getSetPoint(yesterday)
            if setPoint:
                return setPoint

        return None


class DayConfig():

    def __init__(self, days, setPointConfigs):
        self.days = days
        self.setPointConfigs = setPointConfigs

    def getSetPoint(self, dt):
        weekday = dt.weekday() # Monday = 0, Sunday=6
        if not weekday in self.days:
            return None

        for setPointConfig in self.setPointConfigs:
            setPoint = setPointConfig.getSetPoint(dt)
            if setPoint:
                return setPoint

        return None

class SetPointConfig():

    def __init__(self, time, setPoint):
        hours, minutes = map(int, time.split(':'))
        self.minutesMidnight = hours * 60 + minutes
        self.setPoint = setPoint
        #print("Creating time object {}".format(self.minutesMidnight))

    def __lt__(self, other):
        return self.minutesMidnight < other.minutesMidnight

    def getSetPoint(self, dt):
        t = dt.time()
        #print("Checking time {} against {}".format(t, self.minutesMidnight))
        if (t.hour * 60 + t.minute) >= self.minutesMidnight:
            return self.setPoint

        return None


#t = Timer("{}")
#localNow = datetime.now(timezone.utc).astimezone()
#print(t.getCurrentSetPoint(localNow))
